@@ -266,7 +266,7 @@ class SingleCoil(object):
                 fig.show()
         else:
             raise ValueError("Invalid engine option {pyplot, mayavi, plotly}")
-        return
+        return fig
 
     def plot2d(
         self,
@@ -771,13 +771,15 @@ class Coil(object):
                         )
         return
 
-    def toVTK(self, vtkname, line=True, **kwargs):
+    def toVTK(self, vtkname, line=True, height=0.1, width=0.1, **kwargs):
         """Write entire coil set into a VTK file
 
         Args:
             vtkname (str): VTK filename.
-            line (bool): Save coils as polylines or surfaces. Defaults to True.
-            kwargs (dict): Optional kwargs passed to "polyLinesToVTK" or width/height.
+            line (bool, optional): Save coils as polylines or surfaces. Defaults to True.
+            height (float, optional): Rectangle height when expanded to a finite cross-section. Defaults to 0.1.
+            width (float, optional): Rectangle width when expanded to a finite cross-section. Defaults to 0.1.
+            kwargs (dict): Optional kwargs passed to "polyLinesToVTK" or "meshio.Mesh.write".
         """
         from pyevtk.hl import polyLinesToVTK, gridToVTK
 
@@ -807,24 +809,39 @@ class Coil(object):
                 **kwargs
             )
         else:
-            kwargs.setdefault("width", 0.1)
-            kwargs.setdefault("height", 0.1)
+            import meshio
+
+            points = []
+            hedrs = []
+            currents = []
+            groups = []
+            nums = []
+            start = 0
             for i, icoil in enumerate(self):
-                name = vtkname + "{:d}".format(i)
-                xx, yy, zz = icoil.rectangle(
-                    width=kwargs["width"], height=kwargs["height"]
-                )
-                xx = np.atleast_3d(xx)
-                yy = np.atleast_3d(yy)
-                zz = np.atleast_3d(zz)
-                gridToVTK(
-                    name,
-                    xx,
-                    yy,
-                    zz,
-                    pointData={
-                        "I": icoil.I * np.ones_like(xx),
-                        "Igroup": icoil.group * np.ones_like(xx),
-                    },
-                )
+                # example of meshio.Mesh can be found at https://github.com/nschloe/meshio
+                xx, yy, zz = icoil.rectangle(width=width, height=height)
+                xx = np.ravel(np.transpose(xx[0:4, :]))
+                yy = np.ravel(np.transpose(yy[0:4, :]))
+                zz = np.ravel(np.transpose(zz[0:4, :]))
+                xyz = np.transpose([xx, yy, zz])
+                points += xyz.tolist()
+                # number of cells is npoints-1
+                ncell = len(xx) // 4 - 1
+                ind = np.reshape(np.arange(4 * ncell + 4) + start, (-1, 4))
+                hedr = [list(ind[j, :]) + list(ind[j + 1, :]) for j in range(ncell)]
+                hedrs += hedr
+                currents += (icoil.I * np.ones(ncell)).tolist()
+                groups += (icoil.group * np.ones(ncell, dtype=int)).tolist()
+                nums += ((i + 1) * np.ones(ncell, dtype=int)).tolist()
+                # update point index number
+                start += len(xx)
+            kwargs.setdefault("cell_data", {})
+            # coil currents
+            kwargs["cell_data"].setdefault("I", [currents])
+            # current groups
+            kwargs["cell_data"].setdefault("group", [groups])
+            # coil index, starting from 1
+            kwargs["cell_data"].setdefault("index", [nums])
+            data = meshio.Mesh(points=points, cells=[("hexahedron", hedrs)], **kwargs)
+            data.write(vtkname)
         return
