@@ -487,7 +487,7 @@ def vmec2focus(
         with open(bnorm_file, "r") as bfile:
             for line in bfile:
                 tmp = line.split()  # BNORM format: m n Bn_sin
-                bm.append(int(tmp[0]))
+                bnp.append(int(tmp[0]))
                 bn.append(int(tmp[1]))
                 bns.append(float(tmp[2]))
         Nbnf = len(bm)
@@ -619,7 +619,7 @@ def read_focus_boundary(filename):
             line_list = line.split()
             n = int(float(line_list[0]))
             m = int(float(line_list[1]))
-            xm.append(m)
+            xnp.append(m)
             xn.append(n)
             rbc.append(float(line_list[2]))
             rbs.append(float(line_list[3]))
@@ -645,7 +645,7 @@ def read_focus_boundary(filename):
             line_list = line.split()
             n = int(float(line_list[0]))
             m = int(float(line_list[1]))
-            xm.append(m)
+            xnp.append(m)
             xn.append(n)
             bnc.append(float(line_list[2]))
             bns.append(float(line_list[3]))
@@ -657,6 +657,53 @@ def read_focus_boundary(filename):
     return boundary
 
 
+def write_focus_boundary(filename, surf, nfp=1, bn=None, **kwargs):
+    """Write the Fourier harmonics down in FOCUS format
+
+    Args:
+        filename (str): File name to be saved.
+        surf (dict): Plasma surface information, containing 'xm', 'xn', 'rbc', 'rbs', 'zbc', zbs'.
+        nfp (int, optional): Number of field periodicity. Defaults to 1.
+        bn (dict, optional): Nonzero Bn information, containing 'xm', 'xn', 'bnc', 'bns'. Defaults to None.
+    """
+    # write Fourier coefficients
+    mn = len(surf["xn"])
+    try:
+        nbn = len(bn["xn"])
+    except TypeError:
+        nbn = 0
+    # write data
+    with open(filename, "w") as fofile:
+        fofile.write("# bmn   bNfp   nbf " + "\n")
+        fofile.write("{:d} \t {:d} \t {:d} \n".format(mn, nfp, nbn))
+        fofile.write("#plasma boundary" + "\n")
+        fofile.write("# n m Rbc Rbs Zbc Zbs" + "\n")
+        for imn in range(mn):
+            fofile.write(
+                "{:4d}  {:4d} \t {:23.15E}  {:23.15E}  {:23.15E}  {:23.15E} \n".format(
+                    surf["xn"][imn],
+                    surf["xm"][imn],
+                    surf["rbc"][imn],
+                    surf["rbs"][imn],
+                    surf["zbc"][imn],
+                    surf["zbs"][imn],
+                )
+            )
+        fofile.write("#Bn harmonics \n")
+        fofile.write("# n m bnc bns \n")
+        if nbn > 0:
+            for imn in range(nbn):
+                fofile.write(
+                    "{:4d}  {:4d} \t {:23.15E}  {:23.15E}  \n".format(
+                        bn["xn"][imn],
+                        bn["xm"][imn],
+                        bn["bnc"][imn],
+                        bn["bns"][imn],
+                    )
+                )
+    return
+
+
 def div0(a, b):
     return np.divide(a, b, out=np.zeros_like(a), where=b != 0)
 
@@ -669,4 +716,140 @@ def biot_savart(pos, xyz, current, dxyz=None):
         return hanson_hirshman(pos, xyz, current)
     else:
         # tangent provided
-        return biot_savart(pos, xyz, current, dxyz)
\ No newline at end of file
+        return biot_savart(pos, xyz, current, dxyz)
+
+
+def rotation_matrix(alpha=0.0, beta=0.0, gamma=0.0, xyz=False):
+    """A genera rotation matrix using yaw, pitch, and roll angles
+
+    Args:
+        alpha (float, optional): The yaw angle (rotating around the z-axis). Defaults to 0.0.
+        beta (float, optional): The pitch angle (rotating around the y-axis). Defaults to 0.0.
+        gamma (float, optional): The roll angle (rotating around the x-axis). Defaults to 0.0.
+        xyz (bool, optional): The rotation order, True: x->y->z; False: z->y->x. Defaults to False
+
+    Returns:
+        R (3x3 matrix): The rotation matrix.
+    """
+    ca = np.cos(alpha)
+    sa = np.sin(alpha)
+    cb = np.cos(beta)
+    sb = np.sin(beta)
+    cc = np.cos(gamma)
+    sc = np.sin(gamma)
+    if xyz:
+        return np.array(
+            [
+                [cb * cc, -cb * sc, sb],
+                [sa * sb * cc + ca * sc, -sa * sb * sc + ca * cc, -sa * cb],
+                [-ca * sb * cc + sa * sc, ca * sb * sc + sa * cc, ca * cb],
+            ]
+        )
+    else:
+        return np.array(
+            [
+                [ca * cb, ca * sb * sc - sa * cc, ca * sb * cc + sa * sc],
+                [sa * cb, sa * sb * sc + ca * cc, sa * sb * cc - ca * sc],
+                [-sb, cb * sc, cb * cc],
+            ]
+        )
+
+
+def rotation_angle(R, xyz=False):
+    """Get the rotation angle from a rotation matrix
+
+    Args:
+        R (3x3 matrix): The rotation matrix.
+        xyz(bool, optional): the rotation scenario, RxRyRz vs RzRyRx. Defaults to False.
+
+    Returns:
+        [alpha, beta, gamma]: The rotation angle around x,y,z-axis (if xyz=True) or z,y,x-axis.
+    """
+    if not np.allclose(R @ R.T, np.identity(3)):
+        raise ValueError("The rotation matrix is not orthonormal!")
+    if xyz:
+        return _rotation_angle_xyz(R)
+    else:
+        return _rotation_angle_zyx(R)
+
+
+def _rotation_angle_zyx(R):
+    tol = 1e-8
+    # cos(beta) = 0 => beta=pi/2 or 3/2*pi; onle aplha+/-gamma is determined
+    if abs(R[0, 0]) < tol and abs(R[1, 0]) < tol:
+        beta = np.arcsin(-R[2, 0])
+        alpha_pm_gamma = np.arctan2(R[1, 2], R[0, 2])
+        alpha = 0
+        gamma = R[2, 0] * alpha_pm_gamma
+    else:
+        # beta from arcsin
+        beta = np.arcsin(-R[2, 0])
+        cb = np.cos(beta)
+        alpha = np.arctan2(cb * R[1, 0], cb * R[0, 0])
+        sp = np.sin(alpha)
+        cp = np.cos(alpha)
+        gamma = np.arctan2(sp * R[0, 2] - cp * R[1, 2], cp * R[1, 1] - sp * R[0, 1])
+        if not np.allclose(rotation_matrix(alpha, beta, gamma, xyz=False), R):
+            # change another possible value for beta
+            beta = np.pi - np.arcsin(-R[2, 0])
+            cb = np.cos(beta)
+            alpha = np.arctan2(cb * R[1, 0], cb * R[0, 0])
+            sp = np.sin(alpha)
+            cp = np.cos(alpha)
+            gamma = np.arctan2(sp * R[0, 2] - cp * R[1, 2], cp * R[1, 1] - sp * R[0, 1])
+    return alpha, beta, gamma
+
+
+def _rotation_angle_xyz(R):
+    tol = 1e-8
+    # cos(beta) = 0 => beta=pi/2 or 3/2*pi; onle aplha-gamma is determined
+    if abs(R[0, 0]) < tol and abs(R[0, 1]) < tol:
+        beta = np.arcsin(R[0, 2])
+        alpha_pm_gamma = np.arctan2(R[1, 0], -R[2, 0])
+        alpha = 0
+        gamma = R[0, 2] * alpha_pm_gamma
+    else:
+        # beta from arcsin
+        beta = np.arcsin(R[0, 2])
+        cb = np.cos(beta)
+        alpha = np.arctan2(-cb * R[1, 2], cb * R[2, 2])
+        gamma = np.arctan2(-cb * R[0, 1], cb * R[0, 0])
+        if not np.allclose(rotation_matrix(alpha, beta, gamma, xyz=True), R):
+            # change another possible value for beta
+            beta = np.pi - np.arcsin(R[0, 2])
+            cb = np.cos(beta)
+            alpha = np.arctan2(-cb * R[1, 2], cb * R[2, 2])
+            gamma = np.arctan2(-cb * R[0, 1], cb * R[0, 0])
+            if not np.allclose(rotation_matrix(alpha, beta, gamma, xyz=True), R):
+                print("something is still wrong.", alpha, beta, gamma)
+    return alpha, beta, gamma
+
+
+def set_axes_equal(ax):
+    """Make axes of 3D plot have equal scale so that spheres appear as spheres,
+    cubes as cubes, etc..  This is one possible solution to Matplotlib's
+    ax.set_aspect('equal') and ax.axis('equal') not working for 3D.
+
+    Input
+      ax: a matplotlib axis, e.g., as output from plt.gca().
+    """
+
+    x_limits = ax.get_xlim3d()
+    y_limits = ax.get_ylim3d()
+    z_limits = ax.get_zlim3d()
+
+    x_range = abs(x_limits[1] - x_limits[0])
+    x_middle = np.mean(x_limits)
+    y_range = abs(y_limits[1] - y_limits[0])
+    y_middle = np.mean(y_limits)
+    z_range = abs(z_limits[1] - z_limits[0])
+    z_middle = np.mean(z_limits)
+
+    # The plot bounding box is a sphere in the sense of the infinity
+    # norm, hence I call half the max range the plot radius.
+    plot_radius = 0.5 * max([x_range, y_range, z_range])
+
+    ax.set_xlim3d([x_middle - plot_radius, x_middle + plot_radius])
+    ax.set_ylim3d([y_middle - plot_radius, y_middle + plot_radius])
+    ax.set_zlim3d([z_middle - plot_radius, z_middle + plot_radius])
+    return
